// © 2023 John Breaux
//! Removes a single [kind](Type) of [`Token`] from a [`TokenStream`]
use super::*;
#[must_use = "iterators are lazy and do nothing unless consumed"]
#[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
pub struct Ignore<'t, T>
where T: TokenStream<'t>
{
    ignore: Type,
    inner: &'t mut T,
}

impl<'t, T> Ignore<'t, T>
where T: TokenStream<'t>
{
    /// Creates a new [Ignore], which ignores the [ignore Type](Type)
    pub fn new(ignore: Type, t: &'t mut T) -> Self { Ignore { ignore, inner: t } }

    /// Gets a mutable reference to the inner [Iterator]
    pub fn inner_mut(&mut self) -> &mut T { self.inner }
}

impl<'t, T> Iterator for Ignore<'t, T>
where T: TokenStream<'t>
{
    type Item = Token<'t>;
    fn next(&mut self) -> Option<Self::Item> {
        let next = self.inner.next()?;
        // Space tokens are greedy, so the next token shouldn't be a Space
        match next.variant() {
            Type::Space => self.next(),
            _ => Some(next),
        }
    }
}

impl<'t, T> TokenStream<'t> for Ignore<'t, T>
where T: TokenStream<'t>
{
    fn context(&self) -> Context { self.inner.context() }
    fn expect(&mut self, expected: Type) -> Result<Self::Item, LexError> {
        self.inner.allow(self.ignore);
        self.inner.expect(expected)
    }

    fn peek(&mut self) -> Self::Item {
        self.inner.allow(self.ignore);
        self.inner.peek()
    }

    fn peek_expect(&mut self, expected: Type) -> Result<Self::Item, LexError> {
        self.inner.allow(self.ignore);
        self.inner.peek_expect(expected)
    }
}
