// © 2023 John Breaux
//! An [`EncodingParser`] builds an [`Encoding`] from a [`TokenStream`]
use super::*;

#[derive(Clone, Debug)]
/// Builds an [Encoding] using [Tokens](crate::Token) from an input [TokenStream]
pub enum EncodingParser {
    Single { width: Option<Width>, dst: Option<PrimaryOperand> },
    Jump { target: Option<JumpTarget> },
    Double { width: Option<Width>, src: Option<PrimaryOperand>, dst: Option<SecondaryOperand> },
    Reflexive { width: Option<Width>, reg: Option<SecondaryOperand> },
}

impl EncodingParser {
    /// Constructs an [Encoding] from this [EncodingParser], filling holes
    /// with the tokenstream
    pub fn parse<'text, T>(self, p: &Parser, stream: &mut T) -> Result<Encoding, ParseError>
    where T: crate::TokenStream<'text> {
        Ok(match self {
            Self::Single { width, dst } => Encoding::Single {
                width: width.unwrap_or_else(|| Width::parse_or_default(p, stream)),
                dst: if let Some(dst) = dst { dst } else { PrimaryOperand::parse(p, stream)? },
            },
            Self::Jump { target } => Encoding::Jump { target: target.unwrap_or(JumpTarget::parse(p, stream)?) },
            Self::Double { width, src, dst } => Encoding::Double {
                width: width.unwrap_or_else(|| Width::parse_or_default(p, stream)),
                src: if let Some(src) = src { src } else { PrimaryOperand::parse(p, stream)? },
                dst: if let Some(dst) = dst { dst } else { SecondaryOperand::parse(p, stream)? },
            },
            Self::Reflexive { width, reg } => {
                let width = width.unwrap_or_else(|| Width::parse(p, stream).unwrap_or_default());
                let reg = if let Some(reg) = reg { reg } else { SecondaryOperand::parse(p, stream)? };
                Encoding::Double { width, src: reg.clone().into(), dst: reg }
            }
        })
    }
}
